//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// itsonlyme: viewSharedFiles

#include "StdAfx.h"
#include "emule.h"
#include "emuledlg.h"
#include "SearchList.h"
#include "KnownFile.h"
#include "PartFile.h"
#include "SharedFilesPage.h"
#include "ClientDetailDialog.h"
#include "PreviewDlg.h"
#include "CommentDialog.h"
#include "IrcWnd.h"
// itsonlyme: virtualDirs
#include "InputBox.h"
#include "PreferencesDlg.h"
// itsonlyme: virtualDirs
#include "ClientList.h"
#include "SharedFileList.h"
#include "DownloadQueue.h"
#include "UploadQueue.h"
#include "UpDownClient.h"
#include "MenuCmds.h"
#include "WebServices.h"
#include "MenuXP.h"//[TPT] - New Menu Styles

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#define MLC_BLEND(A, B, X) ((A + B * (X-1) + ((X+1)/2)) / X)

#define MLC_RGBBLEND(A, B, X) (                   \
	RGB(MLC_BLEND(GetRValue(A), GetRValue(B), X), \
	MLC_BLEND(GetGValue(A), GetGValue(B), X),     \
	MLC_BLEND(GetBValue(A), GetBValue(B), X))     \
)

#define		MP_SFP_REFRESH_ROOT		1
#define		MP_SFP_REFRESH_DIR		2
#define		MP_SFP_DOWNLOAD			3

#define		BLCT_UNKNOWN	0
#define		BLCT_ROOT	1
#define		BLCT_FILE	2
#define		BLCT_DIR	3
#define		BLCT_LOCALFILE	4

struct BLCItem_struct {
	BLCItem_struct ();
	~BLCItem_struct ();
	int				m_iItemType;
	CString			m_fullPath;
	CString			m_name;
	CSearchFile*	m_file;
	CKnownFile*		m_knownFile;
};

BLCItem_struct::BLCItem_struct () {
	m_iItemType = BLCT_UNKNOWN;
	m_file = NULL;
	m_knownFile = NULL;
}

BLCItem_struct::~BLCItem_struct ()
{
	if (m_file)
		delete m_file;
}

//////////////////////////////////////////////////////////////////////////////
// CSharedFilesPage dialog

IMPLEMENT_DYNAMIC(CSharedFilesPage, CResizablePage)

CSharedFilesPage::CSharedFilesPage()
	: CResizablePage(CSharedFilesPage::IDD, 0)
	, m_ctrlTree(theApp.m_iDfltImageListColorFlags)
{
	m_strCaption = GetResString(IDS_SHAREDFILES);
	m_psp.pszTitle = m_strCaption;
	m_psp.dwFlags |= PSP_USETITLE;
	m_htiRoot = NULL;
	m_toolTip = NULL;
	m_client = NULL;
	m_bLocalFiles = false;
}

CSharedFilesPage::~CSharedFilesPage()
{
}

void CSharedFilesPage::DoDataExchange(CDataExchange* pDX)
{
	CResizablePage::DoDataExchange(pDX); 
	DDX_Control(pDX, IDC_EXT_OPTS, m_ctrlTree);
}

BOOL CSharedFilesPage::OnInitDialog(){
	CResizablePage::OnInitDialog();
	InitWindowStyles(this);

	AddAnchor(IDC_EXT_OPTS,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(IDC_WARNING,TOP_LEFT,TOP_RIGHT);

	Localize();

	m_ctrlTree.ModifyStyle(0, TVS_HASLINES | TVS_HASBUTTONS | TVS_LINESATROOT, 0);
	m_toolTip = new CToolTipCtrl;
	m_toolTip->Create(this, TTS_ALWAYSTIP);
	m_toolTip->SetDelayTime(TTDT_AUTOPOP, 10000);
	m_toolTip->SetDelayTime(TTDT_INITIAL, thePrefs.GetToolTipDelay()*1000); 
	m_toolTip->SendMessage(TTM_SETMAXTIPWIDTH, 0, SHRT_MAX); // recognize \n chars!
	m_toolTip->AddTool(&m_ctrlTree);
	m_ctrlTree.SetToolTips(m_toolTip);

	m_iImgRoot = 8;
	m_iImgShDir = 8;
	m_iImgDir = 8;

	CImageList* imageList = m_ctrlTree.GetImageList(TVSIL_NORMAL);
	if (imageList) {					
		m_iImgRoot = imageList->Add(CTempIconLoader("SharedFiles"));			
		m_iImgShDir = imageList->Add(CTempIconLoader("SharedDir"));			
		m_iImgDir = imageList->Add(CTempIconLoader("PREF_FOLDERS"));
	}

	FillTree();
	m_ctrlTree.Expand(m_htiRoot, TVE_EXPAND);
	m_ctrlTree.SendMessage(WM_VSCROLL, SB_TOP);

	return true;
}

BEGIN_MESSAGE_MAP(CSharedFilesPage, CResizablePage)
	ON_WM_DESTROY()
	ON_NOTIFY(NM_DBLCLK, IDC_EXT_OPTS, OnExtOptsDblClick)
	ON_NOTIFY(TVN_ITEMEXPANDING, IDC_EXT_OPTS, OnItemExpanding)
	ON_NOTIFY_EX_RANGE(TTN_NEEDTEXT,0,0xFFFF,OnToolTipNotify)
	ON_NOTIFY(NM_RCLICK, IDC_EXT_OPTS, OnExtOptsRightClick)
END_MESSAGE_MAP()


// CSharedFilesPage message handlers
void CSharedFilesPage::Localize(){
	CString buffer;
	if (m_bLocalFiles)
		buffer.Format(GetResString(IDS_LOCAL_SHARED_FILES));
	else if (m_client && theApp.clientlist->IsValidClient(m_client)) {
		if (m_client->GetDeniesShare())
			buffer.Format(GetResString(IDS_DENIESSHARE), m_client->GetUserName());
		else
			buffer.Format(GetResString(IDS_SHAREDFILES2), m_client->GetUserName());
	}
	else
		buffer.Format(GetResString(IDS_SHAREDFILES));
	GetDlgItem(IDC_WARNING)->SetWindowText(buffer);

	m_strCaption = GetResString(IDS_SHAREDFILES);
	m_psp.pszTitle = m_strCaption;

	if (m_htiRoot) {
		CString buffer;
		if (!m_bLocalFiles)
			buffer.Format(GetResString(IDS_USERSSHAREDFILES), m_client->GetUserName());
		else
			buffer.Format(GetResString(IDS_USERSSHAREDFILES), thePrefs.GetUserNick());
		m_ctrlTree.SetItemText(m_htiRoot, buffer);
	}
}

void CSharedFilesPage::OnDestroy()
{
	m_ctrlTree.DeleteAllItems();
	m_ctrlTree.DestroyWindow();
	for (POSITION pos = m_BLCItem_list.GetHeadPosition(); pos != NULL; ){
		BLCItem_struct *BLCItem = m_BLCItem_list.GetNext(pos);
		delete BLCItem;
	}
	m_BLCItem_list.RemoveAll();
	m_HTIs_map.RemoveAll();
	if (m_toolTip) {
		m_toolTip->DestroyToolTipCtrl();
		m_toolTip = NULL;
	}
	CResizablePage::OnDestroy();
}

void CSharedFilesPage::OnExtOptsDblClick(NMHDR *pNMHDR, LRESULT *pResult)
{
	if (m_ctrlTree.GetSelectedItem() == NULL)
		return;
	HTREEITEM hti = m_ctrlTree.GetSelectedItem();

	BLCItem_struct *BLCItem = (BLCItem_struct *)m_ctrlTree.GetUserItemData(hti);
	ASSERT(BLCItem);

	if (!m_client)
		return;	// no client
	if (!theApp.clientlist->IsValidClient(m_client)) {
		m_client = NULL;
		return;	// client disconnected
	}

	if (BLCItem->m_iItemType == BLCT_DIR) 
	{
		if (m_client->SendDirRequest(BLCItem->m_fullPath))
			m_ctrlTree.SetItemText(hti, m_ctrlTree.GetItemText(hti) + GetResString(IDS_GETTING_FILE_LIST));
	}
	else if (BLCItem->m_iItemType == BLCT_FILE) // it's a file
	{
		if (BLCItem->m_file)
			theApp.downloadqueue->AddSearchToDownload(BLCItem->m_file);
		RECT itemRect;
		m_ctrlTree.GetItemRect(hti, &itemRect, false);
		m_ctrlTree.InvalidateRect(&itemRect, false);
	}
}

void CSharedFilesPage::OnExtOptsRightClick(NMHDR *pNMHDR, LRESULT *pResult)
{
	CPoint pt;
	::GetCursorPos(&pt);
	int menuX = pt.x, menuY = pt.y;
	HTREEITEM hItem = GetItemUnderMouse();
	if (!hItem) return;
	m_ctrlTree.Select(hItem, TVGN_CARET);
	if (m_ctrlTree.GetItemData(hItem)) {
		BLCItem_struct *BLCItem = (BLCItem_struct *) m_ctrlTree.GetUserItemData(hItem);
		if (BLCItem) {
			//[TPT] - New Menu Styles BEGIN
			CMenuXP	*pVirtualMenu = new CMenuXP;
			pVirtualMenu->CreatePopupMenu();
			pVirtualMenu->SetMenuStyle(CMenuXP::STYLE_STARTMENU);
			
			CMenuXP	*pPrioMenu = new CMenuXP;
			pPrioMenu->CreatePopupMenu();
			pPrioMenu->SetMenuStyle(CMenuXP::STYLE_STARTMENU);

			CMenuXP	*pPermMenu = new CMenuXP;
			pPermMenu->CreatePopupMenu();
			pPermMenu->SetMenuStyle(CMenuXP::STYLE_STARTMENU);

			CMenuXP	*pVirtualDirMenu = new CMenuXP;
			pVirtualDirMenu->CreatePopupMenu();
			pVirtualDirMenu->SetMenuStyle(CMenuXP::STYLE_STARTMENU);	

			CMenuXP *pWebMenu= new CMenuXP;
			pWebMenu->CreatePopupMenu();
			pWebMenu->SetMenuStyle(CMenuXP::STYLE_STARTMENU);
			
			CString buffer = GetResString(IDS_CMT_REFRESH);
			buffer.Remove('&');
			if (BLCItem->m_iItemType == BLCT_ROOT) {
				// itsonlyme: virtualDirs
				if (m_bLocalFiles) {					
					pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_IOM_VIRTPREFS, GetResString(IDS_VDS_ADVANCED)));					
				}
				// itsonlyme: virtualDirs
				else {
					pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_SFP_REFRESH_ROOT, GetResString(IDS_REFRESH_ROOT)));
				}				
			}
			else if (BLCItem->m_iItemType == BLCT_DIR) {
				// itsonlyme: virtualDirs
				if (m_bLocalFiles) {
					pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_IOM_VIRTPREFS, GetResString(IDS_VDS_ADVANCED)));
				}
				// itsonlyme: virtualDirs
				else {
					pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_SFP_REFRESH_DIR, GetResString(IDS_REFRESH_DIR)));
				}				
			}
			else if (BLCItem->m_iItemType == BLCT_FILE) {
				if (thePrefs.IsExtControlsEnabled())					
					pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_DETAIL, GetResString(IDS_SHOWDETAILS)));				
				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_SFP_DOWNLOAD, GetResString(IDS_SW_DOWNLOAD)));				
				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PREVIEW, GetResString(IDS_DL_PREVIEW)));
				pVirtualMenu->AppendSeparator();								
				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_GETED2KLINK, GetResString(IDS_DL_LINK1)));				
				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_GETHTMLED2KLINK, GetResString(IDS_DL_LINK2)));				
			}
			else if (BLCItem->m_iItemType == BLCT_LOCALFILE) {				
				CKnownFile* file = BLCItem->m_knownFile;
				// add priority switcher		
				pPrioMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PRIOVERYLOW, GetResString(IDS_PRIOVERYLOW)));				
				pPrioMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PRIOLOW, GetResString(IDS_PRIOLOW)));				
				pPrioMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PRIONORMAL, GetResString(IDS_PRIONORMAL)));				
				pPrioMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PRIOHIGH, GetResString(IDS_PRIOHIGH)));				
				pPrioMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PRIOVERYHIGH, GetResString(IDS_PRIORELEASE)));				
				pPrioMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PRIOAUTO, GetResString(IDS_PRIOAUTO)));				
				
				UINT uCurPrioMenuItem = 0;
				if (file->IsAutoUpPriority())
					uCurPrioMenuItem = MP_PRIOAUTO;
				else if (file->GetUpPriority() == PR_VERYLOW)
					uCurPrioMenuItem = MP_PRIOVERYLOW;
				else if (file->GetUpPriority() == PR_LOW)
					uCurPrioMenuItem = MP_PRIOLOW;
				else if (file->GetUpPriority() == PR_NORMAL)
					uCurPrioMenuItem = MP_PRIONORMAL;
				else if (file->GetUpPriority() == PR_HIGH)
					uCurPrioMenuItem = MP_PRIOHIGH;
				else if (file->GetUpPriority() == PR_VERYHIGH)
					uCurPrioMenuItem = MP_PRIOVERYHIGH;
				else
					ASSERT(0);
				
				pPrioMenu->CheckMenuRadioItem(MP_PRIOVERYLOW, MP_PRIOAUTO, uCurPrioMenuItem, 0);
				
				// [TPT] - xMule_MOD: showSharePermissions	
				pPermMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PERMNONE, GetResString(IDS_HIDDEN))); // xMule_MOD: showSharePermissions				
				pPermMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PERMFRIENDS, GetResString(IDS_FSTATUS_FRIENDSONLY)));				
				pPermMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_PERMALL, GetResString(IDS_FSTATUS_PUBLIC)));				
				UINT uCurPermMenuItem = 0;
				if (file->GetPermissions() == PERM_ALL)
					uCurPermMenuItem = MP_PERMALL;
				else if (file->GetPermissions() == PERM_FRIENDS)
					uCurPermMenuItem = MP_PERMFRIENDS;
				else if (file->GetPermissions() == PERM_NOONE)
					uCurPermMenuItem = MP_PERMNONE;
				else
					ASSERT(0);
				pPermMenu->CheckMenuRadioItem(MP_PERMALL, MP_PERMNONE, uCurPermMenuItem, 0);
				// [TPT] - xMule_MOD: showSharePermissions	

				// [TPT] - itsonlyme: virtualDirs
				pVirtualDirMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_IOM_VIRTFILE, GetResString(IDS_VDS_MFILE)));				
				pVirtualDirMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_IOM_VIRTDIR, GetResString(IDS_VDS_MDIR)));				
				pVirtualDirMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_IOM_VIRTSUBDIR, GetResString(IDS_VDS_MSDIR)));				
				pVirtualDirMenu->AppendSeparator();				
				pVirtualDirMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_IOM_VIRTREMOVE, GetResString(IDS_VDS_REMOVE)));				
				pVirtualDirMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_IOM_VIRTPREFS, GetResString(IDS_VDS_ADVANCED)));				
				
				bool bVirtRemove = false;
				CString virt, fileID;
				fileID.Format("%i:%s", file->GetFileSize(), EncodeBase16(file->GetFileHash(),16));
				bVirtRemove = thePrefs.GetFileToVDirMap()->Lookup(fileID, virt);
				if (!bVirtRemove) {
					CString path = file->GetPath();
					path.MakeLower();
					path.TrimRight("\\");
					bVirtRemove = thePrefs.GetDirToVDirMap()->Lookup(fileID, virt);
					if (!bVirtRemove)
						bVirtRemove = thePrefs.GetSubDirToVDirMap()->Lookup(fileID, virt);
				}
				pVirtualDirMenu->EnableMenuItem(MP_IOM_VIRTREMOVE, MF_BYCOMMAND | (bVirtRemove ? MF_ENABLED : MF_GRAYED));
				// [TPT] - itsonlyme: virtualDirs

				//menu.AddMenuTitle(GetResString(IDS_FILE));				
				pVirtualMenu->AppendODPopup(MF_STRING|MF_POPUP, pPermMenu, new CMenuXPText(0,GetResString(IDS_PERMISSION))); // xMule_MOD: showSharePermissions - done				
				pVirtualMenu->AppendODPopup(MF_STRING|MF_POPUP, pPrioMenu, new CMenuXPText(0,GetResString(IDS_PRIORITY)));				
				pVirtualMenu->AppendODPopup(MF_STRING|MF_POPUP, pVirtualDirMenu, new CMenuXPText(0,GetResString(IDS_VDS_VIRTDIRTITLE))); // itsonlyme: virtualDirs					
				pVirtualMenu->AppendSeparator();				

				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_DETAIL, GetResString(IDS_SHOWDETAILS)));				
				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_CMT, GetResString(IDS_CMT_ADD)));				
				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_VIEWFILECOMMENTS, GetResString(IDS_CMT_SHOWALL))); // SLUGFILLER: showComments				
				pVirtualMenu->AppendSeparator();				

				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_GETED2KLINK, GetResString(IDS_DL_LINK1)));						
				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_GETHTMLED2KLINK, GetResString(IDS_DL_LINK2)));						
				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_GETSOURCEED2KLINK, GetResString(IDS_CREATESOURCELINK)));						
				pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(MP_GETHOSTNAMESOURCEED2KLINK, GetResString(IDS_CREATEHOSTNAMESRCLINK))); // itsonlyme: hostnameSource				
				pVirtualMenu->AppendSeparator();	

				//This menu option is is for testing..
				if(theApp.emuledlg->ircwnd->IsConnected()){
					pVirtualMenu->AppendODMenu(MF_STRING, new CMenuXPText(Irc_SetSendLink, GetResString(IDS_IRC_ADDLINKTOIRC)));						
					pVirtualMenu->AppendSeparator();	
				}

				// itsonlyme: hostnameSource
				if (theApp.IsConnected() && !theApp.IsFirewalled() &&
					!CString(thePrefs.GetYourHostname()).IsEmpty() &&
					CString(thePrefs.GetYourHostname()).Find(_T(".")) != -1)
					pVirtualMenu->EnableMenuItem(MP_GETHOSTNAMESOURCEED2KLINK, MF_ENABLED);					
				else
					pVirtualMenu->EnableMenuItem(MP_GETHOSTNAMESOURCEED2KLINK, MF_GRAYED);					
				// itsonlyme: hostnameSource				
				pWebMenu->SetSelectedBarColor(RGB(242,120,114));
				int iWebMenuEntries = theWebServices.GetAllMenuEntries(pWebMenu);
				UINT flag2 = (iWebMenuEntries == 0 ) ? MF_GRAYED : MF_STRING;				
				pVirtualMenu->AppendODPopup(flag2|MF_POPUP, pWebMenu, new CMenuXPText(0,GetResString(IDS_WEBSERVICES)));

				// itsonlyme: displayOptions
			}
			pVirtualMenu->TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, menuX, menuY, this);

			delete pWebMenu;
			delete pPrioMenu;
			delete pPermMenu;
			delete pVirtualDirMenu;
			delete pVirtualMenu;
			//[TPT] - New Menu Styles
		}
	}
}

//[TPT] - New Menu Styles BEGIN
void CSharedFilesPage::OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMeasureItemStruct) 
{
	HMENU hMenu = AfxGetThreadState()->m_hTrackingMenu;
	CMenu	*pMenu = CMenu::FromHandle(hMenu);
	pMenu->MeasureItem(lpMeasureItemStruct);
	
	CWnd::OnMeasureItem(nIDCtl, lpMeasureItemStruct);
}
//[TPT] - New Menu Styles END

void CSharedFilesPage::OnItemExpanding (NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);
	HTREEITEM hti = pNMTreeView->itemNew.hItem;

	*pResult = 0;

	if (!m_client)
		return;	// no client
	if (!theApp.clientlist->IsValidClient(m_client)) {
		m_client = NULL;
		return;	// client disconnected
	}

	BLCItem_struct *BLCItem = (BLCItem_struct *) m_ctrlTree.GetUserItemData(hti);
	if (BLCItem->m_iItemType == BLCT_ROOT) {
		m_client->RequestSharedFileList();
		return;
	}
	if (BLCItem->m_iItemType != BLCT_DIR)
		return;

	// send dir list request
	if (m_client->SendDirRequest(BLCItem->m_fullPath))
		m_ctrlTree.SetItemText(hti, m_ctrlTree.GetItemText(hti) + GetResString(IDS_GETTING_FILE_LIST));
	return;
}

BOOL CSharedFilesPage::OnCommand(WPARAM wParam, LPARAM lParam)
{
	HTREEITEM hItem = m_ctrlTree.GetSelectedItem();
	if (hItem == NULL) return false;
	if (!m_ctrlTree.GetItemData(hItem)) return false;
	BLCItem_struct *BLCItem = (BLCItem_struct *) m_ctrlTree.GetUserItemData(hItem);
	if (!BLCItem) return false;
	switch (wParam) {
		case MP_SFP_REFRESH_ROOT: {
			if (BLCItem->m_iItemType == BLCT_ROOT) {
				if (m_client && theApp.clientlist->IsValidClient(m_client)) {
					m_client->SetDeniesShare(false);
					m_client->RequestSharedFileList(true);
					Localize();
				}
			}
			break;
		}
		case MP_SFP_REFRESH_DIR: {
			if (BLCItem->m_iItemType == BLCT_DIR) {
				if (m_client && theApp.clientlist->IsValidClient(m_client)) {
					m_ctrlTree.SetItemText(hItem, m_ctrlTree.GetItemText(hItem) + GetResString(IDS_GETTING_FILE_LIST));
					m_client->SendDirRequest(BLCItem->m_fullPath, true);
				}
			}
			break;
		}
		case MP_SFP_DOWNLOAD: {
			if (BLCItem->m_iItemType == BLCT_FILE) {
				if (BLCItem->m_file)
					theApp.downloadqueue->AddSearchToDownload(BLCItem->m_file);
				RECT itemRect;
				m_ctrlTree.GetItemRect(hItem, &itemRect, false);
				m_ctrlTree.InvalidateRect(&itemRect, false);
			}
			break;
		}
		case MPG_ALTENTER:
		case MP_DETAIL: {
			if (BLCItem->m_iItemType == BLCT_FILE) {
				if (BLCItem->m_file) {
					CKnownFile *file = theApp.downloadqueue->GetFileByID(BLCItem->m_file->GetFileHash());
					if (!file)
						file = theApp.sharedfiles->GetFileByID(BLCItem->m_file->GetFileHash());
					if (!file) { // search file 
						BLCItem->m_file->OpenDetailSheet();	// SLUGFILLER: modelessDialogs
					}
					else if (file->IsPartFile()) { // download file
						CSimpleArray<CPartFile*> aFiles;
						aFiles.Add((CPartFile*)file);
						((CPartFile*)file)->OpenDetailDialog(&aFiles);	// SLUGFILLER: modelessDialogs
					}
					else { // shared file
						CTypedPtrList<CPtrList, CKnownFile*> aFiles;
						aFiles.AddHead(file);
						file->OpenDetailsSheet(aFiles);	// SLUGFILLER: modelessDialogs
					}
				}
			}
			else if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
				CTypedPtrList<CPtrList, CKnownFile*> aFiles;
				aFiles.AddHead(BLCItem->m_knownFile);
				BLCItem->m_knownFile->OpenDetailsSheet(aFiles);	// SLUGFILLER: modelessDialogs
			}
			break;
		}
		case Irc_SetSendLink:
		{
			if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
				theApp.emuledlg->ircwnd->SetSendFileString(CreateED2kLink(BLCItem->m_knownFile));
			}
			break;
		}
		case MP_GETED2KLINK: {
			if (BLCItem->m_iItemType == BLCT_FILE && BLCItem->m_file) {
				CString clpbrd = CreateED2kLink(BLCItem->m_file);
				theApp.CopyTextToClipboard(clpbrd);
			}
			else if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
				CString clpbrd = CreateED2kLink(BLCItem->m_knownFile);
				theApp.CopyTextToClipboard(clpbrd);
			}
			break;
		}
		case MP_GETHTMLED2KLINK: {
			if (BLCItem->m_iItemType == BLCT_FILE && BLCItem->m_file) {
				CString clpbrd = CreateHTMLED2kLink(BLCItem->m_file);
				theApp.CopyTextToClipboard(clpbrd);
			}
			else if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
				CString clpbrd = CreateHTMLED2kLink(BLCItem->m_knownFile);
				theApp.CopyTextToClipboard(clpbrd);
			}
			break;
		}
		case MP_GETSOURCEED2KLINK:
		{
			if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
				CString strLink = theApp.CreateED2kSourceLink(BLCItem->m_knownFile);
				theApp.CopyTextToClipboard(strLink);
			}
			break;
		}
		// itsonlyme: hostnameSource
		case MP_GETHOSTNAMESOURCEED2KLINK:
		{
			if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
				CString strLink = theApp.CreateED2kHostnameSourceLink(BLCItem->m_knownFile);
				theApp.CopyTextToClipboard(strLink);
			}
			break;
		}
		// itsonlyme: hostnameSource
		case MP_PREVIEW:{
			if (BLCItem->m_iItemType == BLCT_FILE && BLCItem->m_file && m_client) {
				CSearchFile *file = BLCItem->m_file;
				CSearchFile *clientFile = NULL;
				for (POSITION pos = m_client->GetListFiles()->GetHeadPosition(); pos != NULL; ) {
					CSearchFile *item = m_client->GetListFiles()->GetNext(pos);
					if (!md4cmp(file->GetFileHash(), item->GetFileHash())) {
						clientFile = item;
						break;
					}
				}
				if (clientFile) {
					if (clientFile->GetPreviews().GetSize() > 0){
						(new PreviewDlg())->SetFile(clientFile);
					}
					else {
						m_client->SendPreviewRequest(clientFile);
						theApp.AddLogLine(true, GetResString(IDS_PREVIEWREQUESTED));
					}
				}
				else
					ASSERT(false); // there should be a corresponding cached CSearchFile* in CUpDownClient*
			}
			break;
		}
		//For Comments 
		case MP_CMT: 
		{ 
			if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
				CCommentDialog dialog(BLCItem->m_knownFile); 
				dialog.DoModal();
			}
			break; 
		} 
		// SLUGFILLER: showComments
		case MP_VIEWFILECOMMENTS:{
			if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
				CTypedPtrList<CPtrList, CKnownFile*> aFiles;
				aFiles.AddHead(BLCItem->m_knownFile);
				BLCItem->m_knownFile->OpenDetailsSheet(aFiles, true);	// SLUGFILLER: modelessDialogs
			}
			break;
		}
		// SLUGFILLER: showComments
		//*END Comments*//

		case MP_PRIOVERYLOW:
			{
				if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
					BLCItem->m_knownFile->SetAutoUpPriority(false);
					BLCItem->m_knownFile->SetUpPriority(PR_VERYLOW);
				}
				break;
			}
		case MP_PRIOLOW:
			{
				if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
					BLCItem->m_knownFile->SetAutoUpPriority(false);
					BLCItem->m_knownFile->SetUpPriority(PR_LOW);
				}
				break;
			}
		case MP_PRIONORMAL:
			{
				if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
					BLCItem->m_knownFile->SetAutoUpPriority(false);
					BLCItem->m_knownFile->SetUpPriority(PR_NORMAL);
				}
				break;
			}
		case MP_PRIOHIGH:
			{
				if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
					BLCItem->m_knownFile->SetAutoUpPriority(false);
					BLCItem->m_knownFile->SetUpPriority(PR_HIGH);
				}
				break;
			}
		case MP_PRIOVERYHIGH:
			{
				if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
					BLCItem->m_knownFile->SetAutoUpPriority(false);
					BLCItem->m_knownFile->SetUpPriority(PR_VERYHIGH);
				}
				break;
			}
		case MP_PRIOAUTO:
			{
				if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
					BLCItem->m_knownFile->SetAutoUpPriority(true);
					BLCItem->m_knownFile->UpdateAutoUpPriority();
				}
				break;
			}		
		// xMule_MOD: showSharePermissions
		case MP_PERMNONE:
			{
				if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
					BLCItem->m_knownFile->SetPermissions(PERM_NOONE);
				}
				break;
			}
		case MP_PERMFRIENDS:
			{
				if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
					BLCItem->m_knownFile->SetPermissions(PERM_FRIENDS);
				}
				break;
			}
		case MP_PERMALL:
			{
				if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
					BLCItem->m_knownFile->SetPermissions(PERM_ALL);
				}
				break;
			}
		// xMule_MOD: showSharePermissions
		// itsonlyme: virtualDirs
		case MP_IOM_VIRTFILE:
		case MP_IOM_VIRTDIR: 
		case MP_IOM_VIRTSUBDIR: {
			InputBox input;
			CString title;
			CKnownFile *file = BLCItem->m_knownFile;
			switch (wParam) {
				case MP_IOM_VIRTFILE:
					title.Format(GetResString(IDS_VDS_CHANGEMAP), file->GetFileName());
					input.SetLabels(title,GetResString(IDS_VDS_VIRTUALFILE),file->GetPath(true));
					break;
				case MP_IOM_VIRTDIR:
					title.Format(GetResString(IDS_VDS_CHANGEMAP), file->GetPath());
					input.SetLabels(title,GetResString(IDS_VDS_VIRTUALDIR),file->GetPath(true));
					break;
				case MP_IOM_VIRTSUBDIR:
					title.Format(GetResString(IDS_VDS_CHANGEMAP), file->GetPath());
					input.SetLabels(title,GetResString(IDS_VDS_VIRTUALSUBDIR),file->GetPath(true));
					break;
			}
			input.DoModal();
			CString output = input.GetInput();
			if (!input.WasCancelled() && output.GetLength()>0) {
				output.MakeLower();
				output.TrimRight("\\");
				CString fileID;
				CString path = file->GetPath();
				path.MakeLower();
				path.TrimRight("\\");
				fileID.Format("%i:%s", file->GetFileSize(), EncodeBase16(file->GetFileHash(),16));
				if (wParam == MP_IOM_VIRTFILE)
					thePrefs.GetFileToVDirMap()->SetAt(fileID, output);
				else if (wParam == MP_IOM_VIRTDIR)
					thePrefs.GetDirToVDirMap()->SetAt(path, output);
				else if (wParam == MP_IOM_VIRTSUBDIR)
					thePrefs.GetSubDirToVDirMap()->SetAt(path, output);
			}
			UpdateTree("");
			break;
		}
		case MP_IOM_VIRTREMOVE: {
			CKnownFile *file = BLCItem->m_knownFile;
			CString virt, fileID;
			CString path = file->GetPath();
			path.MakeLower();
			path.TrimRight("\\");
			fileID.Format("%i:%s", file->GetFileSize(), EncodeBase16(file->GetFileHash(),16));
			if (thePrefs.GetFileToVDirMap()->Lookup(fileID, virt))
				thePrefs.GetFileToVDirMap()->RemoveKey(fileID);
			if (thePrefs.GetDirToVDirMap()->Lookup(path, virt))
				thePrefs.GetDirToVDirMap()->RemoveKey(path);
			if (thePrefs.GetSubDirToVDirMap()->Lookup(path, virt))
				thePrefs.GetSubDirToVDirMap()->RemoveKey(path);
			UpdateTree("");
			break;
		}
		case MP_IOM_VIRTPREFS: {
			theApp.emuledlg->preferenceswnd->OpenDialog();	// SLUGFILLER: modelessDialogs
			theApp.emuledlg->preferenceswnd->SetActivePage(&theApp.emuledlg->preferenceswnd->m_wndVirtual);
			break;
		}
		// itsonlyme: virtualDirs
	}
	return true;
}

BOOL CSharedFilesPage::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	LPNMHDR pNmhdr = (LPNMHDR)lParam;

	switch (pNmhdr->code)
	{
		case NM_CUSTOMDRAW:
		{
			LPNMTVCUSTOMDRAW pCustomDraw = (LPNMTVCUSTOMDRAW)lParam;
			switch (pCustomDraw->nmcd.dwDrawStage)
			{
				case CDDS_PREPAINT:
					// Need to process this case and set pResult to CDRF_NOTIFYITEMDRAW, 
					// otherwise parent will never receive CDDS_ITEMPREPAINT notification.
					*pResult = CDRF_NOTIFYITEMDRAW;
					return true;

				case CDDS_ITEMPREPAINT:
					HTREEITEM hItem = (HTREEITEM) pCustomDraw->nmcd.dwItemSpec;
					if (!m_ctrlTree.GetItemData(hItem)) return true;
					BLCItem_struct *BLCItem = (BLCItem_struct *) m_ctrlTree.GetUserItemData(hItem);
					if (BLCItem) {
						if (BLCItem->m_iItemType == BLCT_FILE) {
							CKnownFile *file = theApp.downloadqueue->GetFileByID(BLCItem->m_file->GetFileHash());
							if (!file)
								file = theApp.sharedfiles->GetFileByID(BLCItem->m_file->GetFileHash());
							if (pCustomDraw->nmcd.uItemState == (CDIS_FOCUS | CDIS_SELECTED))
								pCustomDraw->clrTextBk = MLC_RGBBLEND(::GetSysColor(COLOR_HIGHLIGHT), ::GetSysColor(COLOR_WINDOW), 4);
							if (!file)
								pCustomDraw->clrText = ::GetSysColor(COLOR_WINDOWTEXT);
							else if (file->IsPartFile())
								pCustomDraw->clrText = RGB(224, 0, 0);
							else
								pCustomDraw->clrText = RGB(0, 128, 0);
						}
						else if (BLCItem->m_iItemType == BLCT_LOCALFILE) {
							if (pCustomDraw->nmcd.uItemState == (CDIS_FOCUS | CDIS_SELECTED))
								pCustomDraw->clrTextBk = MLC_RGBBLEND(::GetSysColor(COLOR_HIGHLIGHT), ::GetSysColor(COLOR_WINDOW), 4);
							if (BLCItem->m_knownFile->GetPermissions() == PERM_NOONE)
								pCustomDraw->clrText = RGB(240, 0, 0);
							else if (BLCItem->m_knownFile->GetPermissions() == PERM_FRIENDS)
								pCustomDraw->clrText = RGB(208, 128, 0);
							else if (BLCItem->m_knownFile->IsPartFile())
								pCustomDraw->clrText = RGB(0, 0, 192);
						}
					}
					*pResult = CDRF_SKIPDEFAULT;
					return false;
			}
		}
		break;
	}
	return CResizablePage::OnNotify(wParam, lParam, pResult);
}

BOOL CSharedFilesPage::OnToolTipNotify(UINT id, NMHDR *pNMH, LRESULT *pResult)
{
	TOOLTIPTEXT *pText = (TOOLTIPTEXT *)pNMH;
	int control_id = ::GetDlgCtrlID((HWND)pNMH->idFrom);
	if (!control_id)
		return FALSE;

	CString info;
	
	if (control_id == IDC_EXT_OPTS)
	{
		HTREEITEM hItem = GetItemUnderMouse();
		if (hItem != NULL) {
			CTreeOptionsItemData* pItemData = (CTreeOptionsItemData*) m_ctrlTree.GetItemData(hItem);
			if (pItemData) {
				BLCItem_struct *BLCItem = (BLCItem_struct *) pItemData->m_dwItemData;
				if (BLCItem)
					if (BLCItem->m_iItemType == BLCT_FILE) {
						CPartFile *file = theApp.downloadqueue->GetFileByID(BLCItem->m_file->GetFileHash());
						if (file && file->IsPartFile())
							info = file->GetInfoSummary(file);
						else
							info.Format(
								GetResString(IDS_DL_FILENAME)+": %s\n" +
								GetResString(IDS_FD_HASH) +" %s\n" +
								GetResString(IDS_FD_SIZE) +" %s", 
								BLCItem->m_file->GetFileName(),
								EncodeBase16(BLCItem->m_file->GetFileHash(),16), 
								CastItoXBytes(BLCItem->m_file->GetFileSize()));
					}
			}
		}				
	}
	m_strToolTip.ReleaseBuffer();
	m_strToolTip = info;
	pText->lpszText = m_strToolTip.GetBuffer(1);
	pText->hinst = NULL; // we are not using a resource
	PostMessage(WM_ACTIVATE);
	return TRUE;
}

BOOL CSharedFilesPage::PreTranslateMessage(MSG* pMsg)
{
   	if ( pMsg->message == 260 && pMsg->wParam == 13 && GetAsyncKeyState(VK_MENU)<0 ) {
		PostMessage(WM_COMMAND, MPG_ALTENTER, 0);
		return TRUE;
	}
	if (pMsg->message== WM_LBUTTONDOWN || pMsg->message== WM_LBUTTONUP || pMsg->message== WM_MOUSEMOVE) {
		m_toolTip->RelayEvent(pMsg);
	}
	if (pMsg->message == WM_MOUSEMOVE) {
		HTREEITEM hItem = GetItemUnderMouse();
		if (hItem != NULL)
		{
			if (hItem != m_htiOldToolTipItemDown)
			{
				if (m_toolTip->IsWindowVisible())
					m_toolTip->Update();
				m_htiOldToolTipItemDown = hItem;
			}
		}
	}
	return CResizablePage::PreTranslateMessage(pMsg);
}

static int CALLBACK CompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CTreeOptionsItemData* pItemData1 = (CTreeOptionsItemData*) lParam1;
	BLCItem_struct *Item1 = (BLCItem_struct*)pItemData1->m_dwItemData;
	CTreeOptionsItemData* pItemData2 = (CTreeOptionsItemData*) lParam2;
	BLCItem_struct *Item2 = (BLCItem_struct*)pItemData2->m_dwItemData;

	if (Item1->m_iItemType == Item2->m_iItemType)
		return Item1->m_name.CompareNoCase(Item2->m_name);
	if (Item1->m_iItemType == BLCT_ROOT)
		return -1;
	if (Item2->m_iItemType == BLCT_ROOT)
		return 1;
	if (Item1->m_iItemType == BLCT_DIR)
		return -1;
	if (Item2->m_iItemType == BLCT_DIR)
		return 1;
	ASSERT(false);
	return 0;
}

void CSharedFilesPage::SortDirs(HTREEITEM hParent) {
	HTREEITEM hChild = m_ctrlTree.GetChildItem(hParent);
	while (hChild != NULL) {
		SortDirs(hChild);
		hChild = m_ctrlTree.GetNextSiblingItem(hChild);
	}

	TVSORTCB tvs;

	tvs.hParent = hParent;
	tvs.lpfnCompare = CompareProc;

	m_ctrlTree.SortChildrenCB(&tvs);
}

void CSharedFilesPage::FillTree()
{
	CRBMap <CString, bool> *listDirs = NULL;
	CList <CSearchFile *> *listFiles = NULL;
	if (!m_bLocalFiles) {
		if (!m_client)
			return;	// no client
		if (!theApp.clientlist->IsValidClient(m_client)) {
			m_client = NULL;
			return;	// client disconnected
		}
		listDirs = m_client->GetListDirs();
		listFiles = m_client->GetListFiles();
	}
	else {
		GetDirs();
		listDirs = &m_dirsList;
	}

	BLCItem_struct *BLCItem;

	if (!m_htiRoot) {
		CString buffer;
		BLCItem = new BLCItem_struct;
		BLCItem->m_iItemType = BLCT_ROOT;
		m_BLCItem_list.AddTail(BLCItem);
		if (!m_bLocalFiles)
			buffer.Format(GetResString(IDS_USERSSHAREDFILES), m_client->GetUserName());
		else
			buffer.Format(GetResString(IDS_USERSSHAREDFILES), thePrefs.GetUserNick());
		m_htiRoot = m_ctrlTree.InsertGroup(buffer, m_iImgRoot, TVI_ROOT, TVI_LAST, (DWORD)BLCItem);
		m_HTIs_map.SetAt("\\", m_htiRoot);
		TVITEM it;
		it.hItem = m_htiRoot;
		it.mask = TVIF_CHILDREN;
		it.cChildren = 1;
		m_ctrlTree.SetItem(&it);
	}

	for (POSITION pos = listDirs->GetHeadPosition(); pos != NULL; listDirs->GetNext(pos)) {
		int curPos = 0;
		HTREEITEM dirHti;
		HTREEITEM lastNode = m_htiRoot;
		CString newDir = listDirs->GetKeyAt(pos);

		CString resToken = newDir.Tokenize("\\", curPos);
		while (resToken != "")
		{
			CString fullPath = newDir.Left(curPos);
			if (fullPath.Right(1) == "\\") fullPath.Delete(fullPath.GetLength()-1);
			if (!m_HTIs_map.Lookup(CString(fullPath).MakeLower(), dirHti)) {
				BLCItem = new BLCItem_struct;
				BLCItem->m_iItemType = BLCT_DIR;
				BLCItem->m_fullPath = fullPath;
				BLCItem->m_name = resToken;
				m_BLCItem_list.AddTail(BLCItem);
				dirHti = m_ctrlTree.InsertGroup(resToken, m_iImgDir, lastNode, TVI_LAST, (DWORD)BLCItem);
				m_HTIs_map.SetAt(CString(fullPath).MakeLower(), dirHti);
				TVITEM it;
				it.hItem = dirHti;
				it.mask = TVIF_CHILDREN;
				it.cChildren = 1;
				m_ctrlTree.SetItem(&it);
			}
			lastNode = dirHti;
			resToken = newDir.Tokenize("\\", curPos);
		};
		if (newDir.Right(1) == "\\") newDir.Delete(newDir.GetLength()-1);
		if (m_HTIs_map.Lookup(CString(newDir).MakeLower(), dirHti)) {
			m_ctrlTree.SetItemImage(dirHti, m_iImgShDir, m_iImgShDir);
		}
		else
			ASSERT(false);
	}

	if (m_bLocalFiles) {
		for (uint16 i = 0; i<theApp.sharedfiles->GetCount(); i++) {
			CKnownFile* cur_file = theApp.sharedfiles->GetFileByIndex(i);
			if (cur_file->GetPermissions() == PERM_NOONE)
				continue;
			CString path = cur_file->GetPath(true);	// itsonlyme: virtualDirs
			PathRemoveBackslash(path.GetBuffer());
			path.ReleaseBuffer();
			path.MakeLower();
			HTREEITEM dirHti;
			if (m_HTIs_map.Lookup(path, dirHti)) {
				BLCItem = new BLCItem_struct;
				BLCItem->m_iItemType = BLCT_LOCALFILE;
				BLCItem->m_knownFile = cur_file;
				BLCItem->m_name = cur_file->GetFileName();
				m_BLCItem_list.AddTail(BLCItem);
				HTREEITEM newHti = m_ctrlTree.InsertGroup(BLCItem->m_name, GetFileIcon(BLCItem->m_name), dirHti, TVI_LAST, (DWORD)BLCItem);
			}
			else
				ASSERT (false);
		}
	}
	else {
		for (POSITION pos = listFiles->GetHeadPosition(); pos != NULL; listFiles->GetNext(pos)) {
			CString dir = listFiles->GetAt(pos)->GetDirectory();
			HTREEITEM dirHti;
			if (m_HTIs_map.Lookup(CString(dir).MakeLower(), dirHti)) {
				BLCItem = new BLCItem_struct;
				BLCItem->m_iItemType = BLCT_FILE;
				BLCItem->m_file = new CSearchFile(listFiles->GetAt(pos));
				BLCItem->m_name = BLCItem->m_file->GetFileName();
				m_BLCItem_list.AddTail(BLCItem);
				HTREEITEM newHti = m_ctrlTree.InsertGroup(BLCItem->m_name, GetFileIcon(BLCItem->m_name), dirHti, TVI_LAST, (DWORD)BLCItem);
			}
			else
				ASSERT (false);
		}
	}
	SortDirs(m_htiRoot);
}

bool CSharedFilesPage::FilterDirs(CRBMap<CString, bool> *listDirs, HTREEITEM hParent) {
	BLCItem_struct *BLCItem = (BLCItem_struct *) m_ctrlTree.GetUserItemData(hParent);
	if (BLCItem->m_iItemType != BLCT_DIR) {
		POSITION pos = m_BLCItem_list.Find(BLCItem);
		ASSERT(pos != NULL);
		m_BLCItem_list.RemoveAt(pos);
		m_ctrlTree.DeleteItem(hParent);
		delete BLCItem;
		return false;
	}

	bool requested;
	bool hasChildren = false;
	HTREEITEM hChild = m_ctrlTree.GetChildItem(hParent);
	while (hChild != NULL) {
		HTREEITEM hNext = m_ctrlTree.GetNextSiblingItem(hChild);
		hasChildren |= FilterDirs(listDirs, hChild);
		hChild = hNext;
	}
	if (listDirs->Lookup(BLCItem->m_fullPath, requested))
		return true;
	if (hasChildren) {
		m_ctrlTree.SetItemImage(hParent, m_iImgDir, m_iImgDir);	// Not shared, but has shared subdir
		return true;
	}
	POSITION pos = m_BLCItem_list.Find(BLCItem);
	ASSERT(pos != NULL);
	m_BLCItem_list.RemoveAt(pos);
	m_ctrlTree.DeleteItem(hParent);
	m_HTIs_map.RemoveKey(CString(BLCItem->m_fullPath).MakeLower());
	delete BLCItem;
	return false;
}

void CSharedFilesPage::UpdateTree(CString newDir)
{
	CRBMap <CString, bool> *listDirs = NULL;
	if (!m_bLocalFiles) {
		if (!m_client)
			return;	// no client
		if (!theApp.clientlist->IsValidClient(m_client)) {
			m_client = NULL;
			return;	// client disconnected
		}
		listDirs = m_client->GetListDirs();
	}
	else {
		GetDirs();
		listDirs = &m_dirsList;
	}

	SetRedraw(false);
	HTREEITEM hChild = m_ctrlTree.GetChildItem(m_htiRoot);
	while (hChild != NULL) {
		HTREEITEM hNext = m_ctrlTree.GetNextSiblingItem(hChild);
		FilterDirs(listDirs, hChild);
		hChild = hNext;
	}
	if (!newDir.IsEmpty()) {
		HTREEITEM dirHti;
		if (m_HTIs_map.Lookup(CString(newDir).MakeLower(), dirHti)) {
			BLCItem_struct *BLCItem = (BLCItem_struct*)m_ctrlTree.GetUserItemData(dirHti);
			m_ctrlTree.SetItemImage(dirHti, m_iImgShDir, m_iImgShDir);
			m_ctrlTree.SetItemText(dirHti, BLCItem->m_name);
			m_ctrlTree.SetItemState(dirHti, TVIS_EXPANDED, TVIS_EXPANDED);
		}
		else
			ASSERT(false);
	}
	else
		m_ctrlTree.SetItemState(m_htiRoot, TVIS_EXPANDED, TVIS_EXPANDED);
	Localize();
	FillTree();
	SetRedraw(true);
	CClientDetailDialog *dialog = (CClientDetailDialog *)GetParent();
	dialog->SetActivePage(this);
	Invalidate();
	UpdateWindow();
}

void CSharedFilesPage::GetDirs()
{
	ASSERT(m_bLocalFiles);
	CKnownFile* cur_file;
	m_dirsList.RemoveAll();
	for (uint16 i = 0; i<theApp.sharedfiles->GetCount(); i++) {
		cur_file = theApp.sharedfiles->GetFileByIndex(i);
		if (cur_file->GetPermissions() == PERM_NOONE)
			continue;
		CString path = cur_file->GetPath(true);	// itsonlyme: virtualDirs
		path.MakeLower();
		path.TrimRight("\\");
		m_dirsList.SetAt(path, true);
	}
}

HTREEITEM CSharedFilesPage::GetItemUnderMouse()
{
	CPoint pt;
	::GetCursorPos(&pt);
	m_ctrlTree.ScreenToClient(&pt);
	TVHITTESTINFO hit;
	hit.pt = pt;
	HTREEITEM hItem = m_ctrlTree.HitTest(&hit);
	return hItem;
}

int CSharedFilesPage::GetFileIcon(CString filename)
{
	int srcicon = theApp.GetFileTypeSystemImageIdx(filename.MakeLower());
	int dsticon;

	if (m_iconMap.Lookup(srcicon, dsticon))
		return dsticon;

	CImageList* imageList = m_ctrlTree.GetImageList(TVSIL_NORMAL);
	dsticon = imageList->Add(CImageList::FromHandle(theApp.GetSystemImageList())->ExtractIcon(srcicon));
	m_iconMap.SetAt(srcicon, dsticon);
	return dsticon;
}

// itsonlyme: viewSharedFiles
